/*
 * CopyRight:	HangZhou Hikvision System Technology Co., Ltd. All	Right Reserved.
 * FileName:	HRP_AsyncIO.h
 * Desc:		OS Layer Component.
 * Author:		guanguochen
 * Date:		2009-03-30
 * Contact:		guanguochen@hikvision.com.cn
 * History:		Created By guanguochen 2009-03-30
 * */

/*! 
	\file HPR_AsyncIO.h
	\brief HPR Network Address Manager.

	Details.
*/

#ifndef __HPR_ASYNCIO_EX_H__	
#define __HPR_ASYNCIO_EX_H__

#include "HPR_Config.h"
#include "HPR_Types.h"
#include "HPR_Addr.h"

#if defined(OS_WINDOWS)
	#define HPR_INVALID_ASYNCIOQUEUE NULL
	#define HPR_INVALID_ASYNCIO NULL
#elif defined(OS_POSIX)
	#define HPR_INVALID_ASYNCIOQUEUE (HPR_VOIDPTR)-1
	#define HPR_INVALID_ASYNCIO (HPR_VOIDPTR)-1
#else
	#error OS Not Implement Yet.
#endif

/**
 * HPR_AsyncIO_CreateQueue create async io queue.
 * @param void
 * @return NULL fail, else success.
 * @sa HPR_AsyncIO_DestroyQueue()
 */
HPR_DECLARE HPR_HANDLE CALLBACK HPR_AIO_CreateQueueEx(int iThreadNum,HPR_BOOL bUseCbfLock = HPR_FALSE); 
HPR_DECLARE HPR_HANDLE CALLBACK HPR_AsyncIO_CreateQueueEx_New(int iThreadNum);
HPR_DECLARE HPR_HANDLE CALLBACK HPR_AsyncIO_CreateQueueEx2(void);
HPR_DECLARE HPR_INT32  CALLBACK HPR_AsyncIO_UsePreMem(int iIOCount); 

/**
 * HPR_AsyncIO_DestroyQueue destroy async io queue.
 * @param IOQueueHandle io queue handle, created by  HPR_AsyncIO_CreateQueue.
 * @return 0 success, -1 fail.
 * @sa HPR_AsyncIO_CreateQueue()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_DestroyQueueEx(HPR_HANDLE hIOCP);

/**
 * HPR_AsyncIO_BindIOHandleToQueue bind io handle to async io queue.
 * @param IOHandle io handle
 * @param IOQueueHandle io queue handle, created by  HPR_AsyncIO_CreateQueue.
 * @return 0 success, -1 fail.
 * @sa HPR_AsyncIO_BindCallBackToIOHandle()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_BindIOHandleToQueueEx(HPR_HANDLE hIOFd, HPR_HANDLE hIOCP);
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_UnBindIOHandleEx(HPR_HANDLE hIOFd, HPR_HANDLE hIOCP);

/**
 * HPR_AsyncIO_BindCallBackToIOHandle bind callback function to io handle.
 * @param IOHandle io handle.
 * @param CallBackFunc io completed callback.
 * @return 0 success, -1 fail.
 * @sa HPR_AsyncIO_BindIOHandleToQueue()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_BindCallBackToIOHandleEx(HPR_HANDLE hIOFd, HPR_VOID (*CallBackFunc)(HPR_ULONG nErrorCode, HPR_ULONG nNumberOfBytes, HPR_VOIDPTR pUsrData));

HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_PostQueuedCompleteStatusEx(HPR_HANDLE hIOCP, HPR_HANDLE hIOFd,HPR_INT32 iErrorCode,HPR_UINT32 nNumberOfBytesTransfered, HPR_VOIDPTR pUsrData, HPR_VOIDPTR pCallBack = NULL);

/**
 * HPR_AsyncIO_Send send data by io handle use tcp.
 * @param IOHandle io handle 
 * @param pBuffer data pointer
 * @param BytesToSend data length want to send
 * @param pUsrData user defined data
 * @return 0 success, -1 fail
 * @sa HPR_AsyncIO_Recv()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_SendEx(HPR_HANDLE hIOFd, HPR_VOIDPTR pBuffer, HPR_ULONG BytesToSend, HPR_VOIDPTR pUsrData, HPR_VOIDPTR pCallBack = NULL);

/**
 * HPR_AsyncIO_Recv recv data by io handle use tcp
 * @param IOHandle io handle
 * @param pBuffer data pointer
 * @param BufferLen data length want to recv
 * @param pUsrData user defined data
 * @return 0 success,-1 fail.
 * @sa HPR_AsyncIO_Send()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_RecvEx(HPR_HANDLE hIOFd, HPR_VOIDPTR pBuffer, HPR_ULONG nBufferLen, HPR_VOIDPTR pUsrData, HPR_VOIDPTR pCallBack = NULL);

//////////////////////////////////////////////////////////////////////////
/**asyncio accept ex*/
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_AcceptEx(
	HPR_HANDLE hIOFd, 
	HPR_ADDR_T* pPeerAddr, //linux used. windows not used.
	HPR_VOIDPTR pBuffer,   //not used yet.
	HPR_INT32 iBufLen,     //not used yet.
	HPR_VOIDPTR pUsrData, 
	HPR_VOIDPTR pCallBack, 
	const HPR_INT32 iTimeOut = -1 //not used yet.
	);

/**asyncio connect ex*/
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_ConnectEx(
	HPR_HANDLE hIOFd, 
	HPR_ADDR_T* pPeerAddr, 
	HPR_VOIDPTR pBuffer, //not used yet.
	HPR_INT32   iBufLen, //not used yet.
	HPR_VOIDPTR pUsrData, 
	HPR_VOIDPTR pCallBack, 
	const HPR_INT32 iTimeOut = -1//not used yet.
	);
//////////////////////////////////////////////////////////////////////////

/*
 * add by hdj
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_Accept(HPR_HANDLE hIOFd,  HPR_VOIDPTR pBuffer, HPR_SOCK_T sock, HPR_VOIDPTR pUsrData, HPR_VOIDPTR pCallBack = NULL);

HPR_DECLARE HPR_INT32 CALLBACK HPR_GetAcceptExSockaddrs(HPR_VOIDPTR pBuffer, struct sockaddr* plocal, struct sockaddr* premote);
/**
 * HPR_AsyncIO_SendTo send data by io handle use udp.
 * @param IOHandle io handle
 * @param pBuffer data pointer
 * @param BytesToSend data length want to sendto
 * @param pUsrData user defined data
 * @param pToAddr remote address
 * @return
 * @sa HPR_AsyncIO_RecvFrom()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_SendToEx(HPR_HANDLE hIOFd, HPR_VOIDPTR pBuffer, HPR_ULONG nBytesToSend, HPR_VOIDPTR pUsrData, HPR_ADDR_T* pDstAddr, HPR_VOIDPTR pCallBack = NULL);

/**
 * HPR_AsyncIO_RecvFrom recv data by io handle by udp.
 * @param IOHandle io handle
 * @param pBuffer data pointer.
 * @param BufferLen data length want to recv
 * @param pUsrData user defined data
 * @param pFromAddr remote address
 * @return 0 success, -1 fail
 * @sa HPR_AsyncIO_SendTo()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_RecvFromEx(HPR_HANDLE hIOFd, HPR_VOIDPTR pBuffer, HPR_ULONG nBufferLen, HPR_VOIDPTR pUsrData, HPR_ADDR_T* pAddrSrc, HPR_VOIDPTR pCallBack = NULL);

/**
 * HPR_AsyncIO_WriteFile write data to io handle
 * @param IOHandle io handle
 * @param pBuffer data to write
 * @param BytesToWrite data length want to write to handle.
 * @param pUsrData user defined data.
 * @return 0 success, -1 fail.
 * @sa HPR_AsyncIO_ReadFile()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_WriteFileEx(HPR_HANDLE hIOFd, HPR_VOIDPTR pBuffer, HPR_ULONG nBytesToWrite, HPR_VOIDPTR pUsrData, HPR_VOIDPTR pCallBack = NULL);

/**
 * HPR_AsyncIO_ReadFile read data from io handle
 * @param IOHandle io handle
 * @param pBuffer data pointer
 * @param BufferLen data len want to read.
 * @param pUsrData user defined data.
 * @return 0 success, -1 fail.
 * @sa HPR_AsyncIO_WriteFile()
 */
HPR_DECLARE HPR_INT32 CALLBACK HPR_AsyncIO_ReadFileEx(HPR_HANDLE hIOFd, HPR_VOIDPTR pBuffer, HPR_ULONG nBufferLen, HPR_VOIDPTR pUsrData, HPR_VOIDPTR pCallBack = NULL);

typedef HPR_VOID (*IOCALLBACK)(HPR_ULONG nErrorCode, HPR_ULONG nNumberOfBytes, HPR_VOIDPTR pUsrData);

//class HPR_AsyncIOQueue;
//
//class HPR_AsyncIOEx
//{
//public:
//	HPR_AsyncIOEx(HPR_HANDLE IOHandle);
//	~HPR_AsyncIOEx(HPR_VOID);
//
//	HPR_INT32 BindIOHandleToQueue(HPR_AsyncIOQueue *pIOQueue);
//	HPR_INT32 BindCallBackToIOHandle(HPR_VOID (*CallBackFunc)(HPR_ULONG nErrorCode, HPR_ULONG NumberOfBytes, HPR_VOIDPTR pUsrData));
//	HPR_INT32 IOSend(HPR_VOIDPTR pBuffer, HPR_ULONG BytesToSend, HPR_VOIDPTR pUsrData);
//	HPR_INT32 IORecv(HPR_VOIDPTR pBuffer, HPR_ULONG BufferLen, HPR_VOIDPTR pUsrData);
//	HPR_INT32 IOSendTo(HPR_VOIDPTR pBuffer, HPR_ULONG BytesToSend, HPR_VOIDPTR pUsrData, HPR_ADDR_T* pToAddr);
//	HPR_INT32 IORecvFrom(HPR_VOIDPTR pBuffer, HPR_ULONG BufferLen, HPR_VOIDPTR pUsrData, HPR_ADDR_T* pFromAddr);
//	HPR_INT32 IOWriteFile(HPR_VOIDPTR pBuffer, HPR_ULONG BytesToWrite, HPR_VOIDPTR pUsrData);
//	HPR_INT32 IOReadFile(HPR_VOIDPTR pBuffer, HPR_ULONG BufferLen, HPR_VOIDPTR pUsrData);
//
//private:
//	HPR_HANDLE m_iohandle;
//};
//
//class HPR_AsyncIOQueueEx
//{
//public:
//	HPR_AsyncIOQueueEx(HPR_VOID);
//	~HPR_AsyncIOQueueEx(HPR_VOID);
//
//	HPR_INT32 Initialize(HPR_VOID);
//	HPR_INT32 Release(HPR_VOID);
//	HPR_HANDLE GetHandle(HPR_VOID);
//
//private:
//	HPR_HANDLE m_queuehandle;
//};

#endif

